//Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
//
//This file is part of CADNA.
//
//    CADNA is free software: you can redistribute it and/or modify it
//    under the terms of the GNU Lesser General Public License as
//    published by the Free Software Foundation, either version 3 of the
//    License, or (at your option) any later version.
//
//    CADNA is distributed in the hope that it will be useful, but WITHOUT
//    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
//    Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with CADNA. If not, see
//    <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////
//****m* cadna_sub/operator-
//    NAME
//      operator-
//    SYNOPSIS
//      res = a - b 
//    FUNCTION
//      Define all the functions involving at least one argument
//      of stochastic type which overload the "-" operator
//      in a statement such as "a-b".
//    INPUTS
//      a           - an integer, a float, a double or a stochastic number
//      b           - an integer, a float, a double or a stochastic number 
//      At least one argument must be of stochastic type.
//    RESULT
//      res         - a stochastic number
//             
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//  
//    NOTES
//  
//  
//    BUGS
//  
//  
//    SEE ALSO
//      
//      
//  /

#include "cadna.h"
#include  "cadna_private.h"


  float_st float_st::operator-() const			 
  {						 
    float_st res;					 
    res.x=-x;					 
    res.y=-y;					 
    res.z=-z;					 
    res.accuracy = accuracy;			 
    return res;					 
  }
 
 
  double_st double_st::operator-() const			 
  {						 
    double_st res;					 
    res.x=-x;					 
    res.y=-y;					 
    res.z=-z;					 
    res.accuracy = accuracy;			 
    return res;					 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  float_st float_st::operator--()						 
  {									 
    float_st initial=*this;							 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch(); --x;					 
    if (RANDOM) rnd_switch(); --y;					 
    rnd_switch(); --z;							 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st double_st::operator--()						 
  {									 
    double_st initial=*this;							 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch(); --x;					 
    if (RANDOM) rnd_switch(); --y;					 
    rnd_switch(); --z;							 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  float_st float_st::operator--(int)						 
  {									 
    float_st initial=*this;							 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch(); --x;					 
    if (RANDOM) rnd_switch(); --y;					 
    rnd_switch(); --z;							 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return initial;							 
  }
 
 
  double_st double_st::operator--(int)						 
  {									 
    double_st initial=*this;							 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch(); --x;					 
    if (RANDOM) rnd_switch(); --y;					 
    rnd_switch(); --z;							 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return initial;							 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  double_st& double_st::operator-=(const double& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const float& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const unsigned long long& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const long long& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const unsigned long& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const long& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const unsigned int& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const int& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const unsigned short& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const short& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const unsigned char& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const char& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 

  float_st& float_st::operator-=(const double& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const float& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const unsigned long long& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const long long& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const unsigned long& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const long& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const unsigned int& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const int& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const unsigned short& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const short& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const unsigned char& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const char& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x -= a;								 
    if (RANDOM) rnd_switch();						 
    y -= a;								 
    rnd_switch();							 
    z -= a;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (initial.accuracy >=  this->nb_significant_digit()+_cadna_cancel_value)  
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 



/////////////////////////////////////////////////////////////////////////


  double_st& double_st::operator-=(const  double_st& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x-=a.x;								 
    if (RANDOM) rnd_switch();						 
    y-=a.y;								 
    rnd_switch();							 
    z-=a.z;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if ((initial.accuracy < a.accuracy ? initial.accuracy : a.accuracy) >=  
	  nb_significant_digit()+_cadna_cancel_value)			 
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  double_st& double_st::operator-=(const  float_st& a)				 
  {									 
    double_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x-=a.x;								 
    if (RANDOM) rnd_switch();						 
    y-=a.y;								 
    rnd_switch();							 
    z-=a.z;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if ((initial.accuracy < a.accuracy ? initial.accuracy : a.accuracy) >=  
	  nb_significant_digit()+_cadna_cancel_value)			 
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const  double_st& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x-=a.x;								 
    if (RANDOM) rnd_switch();						 
    y-=a.y;								 
    rnd_switch();							 
    z-=a.z;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if ((initial.accuracy < a.accuracy ? initial.accuracy : a.accuracy) >=  
	  nb_significant_digit()+_cadna_cancel_value)			 
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 
  float_st& float_st::operator-=(const  float_st& a)				 
  {									 
    float_st initial=*this;						 
									 
    accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    x-=a.x;								 
    if (RANDOM) rnd_switch();						 
    y-=a.y;								 
    rnd_switch();							 
    z-=a.z;								 
									 
    if (_cadna_cancel_tag){						 
      if (initial.accuracy==DIGIT_NOT_COMPUTED || initial.accuracy==RELIABLE_RESULT)  
	initial.nb_significant_digit();					 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if ((initial.accuracy < a.accuracy ? initial.accuracy : a.accuracy) >=  
	  nb_significant_digit()+_cadna_cancel_value)			 
	instability(&_cadna_cancel_count);				 
    }									 
    return *this;							 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  double_st operator-(const  double& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  float& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  unsigned long long& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  long long& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  unsigned long& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  long& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  unsigned int & a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  int & a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  unsigned short& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  short& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  unsigned char& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  char& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 

  float_st operator-(const  double& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  unsigned long long& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  long long& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  unsigned long& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  long& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  unsigned int & a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  int & a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  unsigned short& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  short& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  unsigned char& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  char& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a-b.y;							 
    rnd_switch();							 
    res.z=a-b.z;							 
									 
    if (_cadna_cancel_tag){						 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if (b.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  double_st operator-(const  double_st& a, const  double& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  float& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  unsigned long long& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  long long& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  unsigned long& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  long& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  unsigned int & b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  int & b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  unsigned short& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  short& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  unsigned char& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const  double_st& a, const  char& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 

  float_st operator-(const  float_st& a, const  double& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  float& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  unsigned long long& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  long long& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  unsigned long& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  long& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  unsigned int & b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  int & b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  unsigned short& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  short& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  unsigned char& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const  float_st& a, const  char& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b;							 
    rnd_switch();							 
    res.z=a.z-b;							 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (a.accuracy >= 	res.nb_significant_digit()+_cadna_cancel_value)	 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 

/////////////////////////////////////////////////////////////////////////


  double_st operator-(const double_st& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b.y;							 
    rnd_switch();							 
    res.z=a.z-b.z;							 
									 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if ((a.accuracy < b.accuracy ? a.accuracy: b.accuracy) >=		 
	  res.nb_significant_digit()+_cadna_cancel_value)		 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const double_st& a, const float_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b.y;							 
    rnd_switch();							 
    res.z=a.z-b.z;							 
									 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if ((a.accuracy < b.accuracy ? a.accuracy: b.accuracy) >=		 
	  res.nb_significant_digit()+_cadna_cancel_value)		 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  double_st operator-(const float_st& a, const double_st& b)			 
  {									 
    double_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b.y;							 
    rnd_switch();							 
    res.z=a.z-b.z;							 
									 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if ((a.accuracy < b.accuracy ? a.accuracy: b.accuracy) >=		 
	  res.nb_significant_digit()+_cadna_cancel_value)		 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 
  float_st operator-(const float_st& a, const float_st& b)			 
  {									 
    float_st res;								 
									 
    res.accuracy=DIGIT_NOT_COMPUTED;					 
    if (RANDOM) rnd_switch();						 
    res.x=a.x-b.x;							 
    if (RANDOM) rnd_switch();						 
    res.y=a.y-b.y;							 
    rnd_switch();							 
    res.z=a.z-b.z;							 
									 
									 
    if (_cadna_cancel_tag){						 
      if (a.accuracy==DIGIT_NOT_COMPUTED || a.accuracy==RELIABLE_RESULT)  
	a.nb_significant_digit();					 
      if (b.accuracy==DIGIT_NOT_COMPUTED || b.accuracy==RELIABLE_RESULT)  
	b.nb_significant_digit();					 
      if ((a.accuracy < b.accuracy ? a.accuracy: b.accuracy) >=		 
	  res.nb_significant_digit()+_cadna_cancel_value)		 
	instability(&_cadna_cancel_count);				 
    }									 
    return res;								 
  }
 
 

/////////////////////////////////////////////////////////////////////////

